#Copyright (c) 2018 Vereign AG [https://www.vereign.com]
#
#This is free software: you can redistribute it and/or modify
#it under the terms of the GNU Affero General Public License as
#published by the Free Software Foundation, either version 3 of the
#License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Affero General Public License for more details.
#
#You should have received a copy of the GNU Affero General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

#https://gaiax.vereign.com/tsa/policy/policy/:group/:name/:version/evaluation
#Author: Georgi Michev georgi.michev@vereign.com

@rest @all @tsa @policy
Feature: API -TSA - Policy - :group/:name/:version/evaluation POST
  As user
  I want to evaluate the policy
  So I am able to execute it in the future

  Acceptance criteria:
  - HTTP endpoint to evaluate the policy
  - example policy created and committed to Git repo
  - Green test based on example committed to the system

  Background:
    Given we are testing the TSA Policy Api

  Scenario: TSA - Evaluate policy synchronously - Positive
    Given I load the REST request {Policy.json} with profile {successful_evaluate}
    And I execute the policy {/policy/example/test/1.0/evaluation}
    Then the status code should be {200}
    And the response is valid according to the {Policy_Evaluate_schema.json} REST schema
    And the field {result.allow} has the value {true}

  @issue-35
  Scenario: TSA - DID resolution - Positive
    Given I load the REST request {Policy.json} with profile {didResolve_evaluate}
    And I execute the policy {/policy/example/resolve/1.0/evaluation}
    Then the status code should be {200}
    And the response is valid according to the {Policy_EvaluateDID_schema.json} REST schema
    And the field {result.data.didDocument.id} has the value {did:key:z6Mkfriq1MqLBoPWecGoDLjguo1sB9brj6wT3qZ5BxkKpuP6}

  @negative
  Scenario: TSA - Evaluate policy with incorrect value - Negative
    Given I load the REST request {Policy.json} with profile {unsuccessful_evaluate}
    And I execute the policy {/policy/example/test/1.0/evaluation}
    Then the status code should be {200}
    And the response is valid according to the {Policy_Evaluate_schema.json} REST schema
    And the field {result.allow} has the value {false}

  @negative
  Scenario: TSA - Evaluate policy with missing body- Negative
    Given I execute the policy {/policy/example/test/1.0/evaluation}
    Then the status code should be {400}
    And the response is valid according to the {Policy_Evaluate_negative_schema.json} REST schema
    And the field {message} has the value {missing required payload}

  @negative
  Scenario: TSA - Evaluate missing policy - Negative
    Then I load the REST request {Policy.json} with profile {successful_evaluate}
    Given I execute the policy {/policy/testable/test0/1.0/evaluation}
    Then the status code should be {404}
    And the response is valid according to the {Policy_Evaluate_negative_schema.json} REST schema
    And the field {message} has the value {error evaluating policy}

  @negative @issue-35
  Scenario: TSA - DID resolution with incorrect did - Negative
    Then I load the REST request {Policy.json} with profile {didResolve_missing_method}
    And I execute the policy {/policy/example/resolve/1.0/evaluation}
    Then the status code should be {200}
    And the response is valid according to the {Policy_EvaluateDID_negative_schema.json} REST schema
    And the field {result.data.didResolutionMetadata.error} has the value {notFound}
